/* --------------------------------------------------------------------------------------------
 * Copyright (c) 2018 TypeFox GmbH (http://www.typefox.io). All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
// import * as fs from "fs";
// import { xhr, getErrorStatusDescription } from 'request-light';
import { URI } from 'vscode-uri';
import { MessageReader, MessageWriter } from "vscode-jsonrpc";
import { _Connection, createConnection } from 'vscode-languageserver/lib/node/main';
import { InitializeParams, InitializeResult } from 'vscode-languageserver/node';
import { VLS } from '../../server/dist/vls';

export function start(reader: MessageReader, writer: MessageWriter): JsonServer {
    const connection = createConnection(reader, writer);
    const server = new JsonServer(connection);
    // server.start();
    return server;
}

export class JsonServer {
  protected workspaceRoot: URI | undefined;

  protected readonly pendingValidationRequests = new Map<string, number>();

  constructor(
    protected readonly connection: _Connection
  ) {
    const vls = new VLS(connection);
    connection.onInitialize(
      async (params: InitializeParams): Promise<InitializeResult> => {
        await vls.init(params);

        console.log('Lowcode View DSL lsp server initialized');
        this.connection.console.log("Lowcode View DSL lsp server initialized");
        return {
          capabilities: vls.capabilities,
        };
      }
    );
    vls.listen();
  }
}
